@@ -1,16 +1,22 @@
-from typing import Any, Callable, Dict
+from typing import Any, Callable, Dict, Union
 
 import numpy as np
-import torch
-
 import thingsvision.custom_models as custom_models
 import thingsvision.custom_models.cornet as cornet
+from torchtyping import TensorType
 
-from .base import BaseExtractor
-from .extractor import KerasExtractor, TimmExtractor, TorchvisionExtractor, VisslExtractor
-from .mixin import PyTorchMixin, TensorFlowMixin
+import torch
+import warnings
+
+from .extractors import (
+    KerasExtractor,
+    SSLExtractor,
+    TimmExtractor,
+    TorchvisionExtractor,
+)
+from .tensorflow import TensorFlowExtractor
+from .torch import PyTorchExtractor
 
-Tensor = torch.Tensor
 Array = np.ndarray
 AxisError = np.AxisError
 
@@ -33,6 +39,7 @@ def create_custom_extractor(
         model = model.module  # remove DataParallel
         preprocess = None
     elif hasattr(custom_models, model_name):
+        model_parameters = model_parameters if model_parameters else {}
         custom_model = getattr(custom_models, model_name)
         custom_model = custom_model(device, model_parameters)
         model, preprocess = custom_model.create_model()
@@ -42,9 +49,9 @@ def create_custom_extractor(
             f"\nCould not find {model_name} among custom models.\nChoose a different model.\n"
         )
 
-    backend_mixin = PyTorchMixin if backend == "pt" else TensorFlowMixin
+    Extractor = PyTorchExtractor if backend == "pt" else TensorFlowExtractor
 
-    class CustomExtractor(BaseExtractor, backend_mixin):
+    class CustomExtractor(Extractor):
         def __init__(self, *args, **kwargs) -> None:
             super().__init__(*args, **kwargs)
 
@@ -58,21 +65,25 @@ def create_custom_extractor(
                         print(n)
             print("visual")
 
-        def forward(self, batch: Tensor, module_name: str = "visual") -> Tensor:
+        @staticmethod
+        def forward(batch: TensorType["b", "c", "h", "w"]) -> TensorType["b", "d"]:
             img_features = model.encode_image(batch)
-            # if module_name == "visual":
-            # assert torch.unique(
-            #    activations[module_name] == img_features
-            # ).item(), "\nFor CLIP, image features should represent activations in last encoder layer.\n"
-
             return img_features
 
-        def flatten_acts(self, act: Tensor, img: Tensor, module_name: str) -> Tensor:
+        @staticmethod
+        def flatten_acts(
+            act: Union[
+                TensorType["b", "num_maps", "h_prime", "w_prime"],
+                TensorType["b", "t", "d"],
+            ],
+            batch: TensorType["b", "c", "h", "w"],
+            module_name: str,
+        ) -> TensorType["b", "p"]:
             if module_name.endswith("attn"):
                 if isinstance(act, tuple):
                     act = act[0]
             else:
-                if act.size(0) != img.shape[0] and len(act.shape) == 3:
+                if act.size(0) != batch.shape[0] and len(act.shape) == 3:
                     act = act.permute(1, 0, 2)
             act = act.view(act.size(0), -1)
             return act
@@ -83,7 +94,9 @@ def create_custom_extractor(
 
     if model_name == "OpenCLIP":
 
-        def forward(self, batch: Tensor) -> Tensor:
+        def forward(
+            self, batch: TensorType["b", "c", "h", "w"]
+        ) -> TensorType["b", "d"]:
             return self.model(batch, text=None)
 
         CustomExtractor.forward = forward
@@ -106,6 +119,7 @@ def create_model_extractor(
     preprocess: Any = None,
     backend: str = "pt",
     forward_fn: Callable = None,
+    flatten_fn: Callable = None,
 ) -> Any:
     """
     Creates a class for extracting activations from a given model (PyTorch or TensorFlow).
@@ -121,33 +135,54 @@ def create_model_extractor(
     backend: str
         The backend of the model. Either "pt" for PyTorch or "tf" for TensorFlow.
     forward_fn: Callable
-        In case your model requires more complicated forward passes than simply using model(img),
-        you can pass a custom forward function here. The function must have the following signature:
+        In case your model requires more complicated forward passes than the default,
+        for example if the forward pass requires multiple inputs, you can pass a custom
+        forward function here, which should take a batch of images as input. Only required
+        for PyTorch models.
 
-        forward_fn(self, img, module_name) -> activations
+        Default:
+            def forward(
+                self, batch: TensorType["b", "c", "h", "w"]
+            ) -> TensorType["b", "num_cls"]:
+                return self.model(batch)
 
-        and calls to the model have to be made on the self.model attribute.
+    flatten_fn: Callable
+        In case your model requires more complicated flattening of activations than
+        the default, you can pass a custom flattening function here. Only required for
+        PyTorch models.
+
+        Default:
+            def flatten_acts(
+                act: Union[
+                    TensorType["b", "num_maps", "h_prime", "w_prime"], TensorType["b", "t", "d"]
+                ]
+            ) -> TensorType["b", "p"]:
+                return act.view(act.size(0), -1)
 
     Returns:
     --------
     extractor: Any
-        The custom extractor class.
+        The custom extractor class for your model.
     """
-    backend_mixin = PyTorchMixin if backend == "pt" else TensorFlowMixin
+    Extractor = PyTorchExtractor if backend == "pt" else TensorFlowExtractor
 
-    class ModelExtractor(BaseExtractor, backend_mixin):
+    class ModelExtractor(Extractor):
         def __init__(self, *args, **kwargs):
             super().__init__(*args, **kwargs)
 
     if forward_fn:
         ModelExtractor.forward = forward_fn
 
+    if flatten_fn:
+        ModelExtractor.flatten_acts = flatten_fn
+
     model_extractor = ModelExtractor(
         model_name="custom",
         model_path=None,
         device=device,
         model=model,
         preprocess=preprocess,
+        pretrained=True,
     )
 
     return model_extractor
@@ -178,23 +213,33 @@ def get_extractor(
         return KerasExtractor(**model_args)
     elif source == "custom":
         return create_custom_extractor(**model_args)
+    elif source == "ssl":
+        return SSLExtractor(**model_args)
     elif source == "vissl":
-        return VisslExtractor(**model_args)
+        warnings.warn(
+            message='\nThe source "vissl" is deprecated. Use the source "ssl" instead.\n',
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
+        return SSLExtractor(**model_args)
     else:
         raise ValueError(
-            f"\nCould not find {source} library.\nChoose a different source.\n"
+            f"\nCould not find source: {source}.\nChoose a different source.\n"
         )
 
 
 def get_extractor_from_model(
     model: Any,
     device: str,
+    backend: str,
     preprocess: Any = None,
-    backend: str = "pt",
     forward_fn: Callable = None,
+    flatten_fn: Callable = None,
 ) -> Any:
     """Get a model extractor from a model."""
-    return create_model_extractor(model, device, preprocess, backend, forward_fn)
+    return create_model_extractor(
+        model, device, preprocess, backend, forward_fn, flatten_fn
+    )
 
 
 def center_features(X: Array) -> Array:
