@@ -5,4 +5,4 @@ from .vgg16_ecoset import VGG16_ecoset
 from .openclip import OpenCLIP
 from .inception_ecoset import Inception_ecoset
 from .official_clip import clip
-# from .harmonization import Harmonization
+from .harmonization import Harmonization
